#!/usr/bin/python

# RSS Mix Tape
# Copyright 2003, David "Novalis" Turner <novalis@fsf.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from __future__ import generators

from bsddb import *
import time
import os

import DateTime

import rssparser

import config
from Post2 import Post

class Feed:
    def __init__ (self, title, interval, url, lastchecked = 0, id = -1):
        """Create a new feed"""
        self.id = id
        if id == -1:
            self.id = config.counter ()
        self.title = title
        self.interval = int (interval)
        self.url = url
        self.lastchecked = int (lastchecked)
        
        self.recentPosts = None
        self.listener = None
        self.posts = None
        self.db = None
        
    def toValue (self):
        return "\0".join ([self.title, str (self.interval), self.url,
                           str (self.lastchecked)])


    def setListener (self, listener):
        self.listener = listener

    def open (self):
        if self.db is not None:
            return
        
        if not os.path.exists (config.dir + "feed/"):
            os.mkdir (config.dir + "feed/")

        self.db = btopen (config.dir + "feed/" + str (self.id) + ".db", "c")

    def update (self):
        """Updates the data in the feed from the url"""

        self.open ()

        #get the 15 most recent posts, if we haven't already got them
        if not self.recentPosts:
            i = 0
            self.recentPosts = []
            try:
                k, v = self.db.last ()
                self.recentPosts.append (self.createPost (k, v))
                while 1:
                    if i >= 15:
                        break
                    k, v = self.db.previous () 
                    self.recentPosts.append (self.createPost (k, v))
            except KeyError:
                pass
            
        self.getPostsFromRSS ()


    def getPostsFromRSS (self):
        rss = rssparser.parse (self.url)
        items = rss ["items"]
        channel = rss ["channel"]

        now = int (time.time ())

        for item in items:
            self.feedItem (item, channel, now)
                
        self.lastchecked = now

    def feedItem (self, item, channel, now):
        if not 'date' in item:
            if 'date' in channel:
                item ['date'] = channel ['date']
            else:
                if 'lastBuildDate' in channel:
                    item ['date'] = channel ['lastBuildDate']
                else:
                    item ['date'] = time.asctime (time.localtime (now))

        date = int (DateTime.DateTime (item ['date']).timeTime ())

        if not 'title' in item:
            if 'description' in item:
                item ['title'] = item ['description']
            else: 
                if 'title' in channel:
                    item ['title'] = channel ['title']
                else:
                    item ['title'] = 'untitled'

        if len (item ['title']) > 50:
            item ['title'] = item ['title'][0:50] + "..."

        if not 'text' in item:
            if not 'description' in item:
                item ['text'] = ''
            else:
                item ['text'] = item ['description']

        post = Post (self.title, date, item ['title'], item ['link'], item ['text'])
        self.addPost (post)

    def sync (self):
        if self.posts:
            for post in self.posts:
                self.db [str (post.id)] = post.toValue ()
        self.db.sync ()

    def close (self):
        if self.db is not None:
            if self.posts:
                for post in self.posts:
                    self.db [str (post.id)] = post.toValue ()
            self.db.close ()
            self.db = None

    def readPosts (self):
        if self.posts:
            for post in self.posts:
                self.listener (post)
            return
                
        self.posts = []
        try:
            k, v = self.db.first ()
            self.addPost (self.createPost (k, v), 0)
            while 1:
                k, v = self.db.next ()
                self.addPost (self.createPost (k, v), 0)
        except KeyError:
            return
        
    def forPosts (self, fun):
        if not self.posts:
            self.readPosts ()
        for post in self.posts:
            fun (post)

    def addPost (self, post, check = 1):
        if not self.recentPosts:
            self.recentPosts = []
            
        if check:
            for oldpost in self.recentPosts:
                if oldpost.title == post.title and oldpost.text == post.text \
                       and oldpost.date == post.date:
                    #seen this one
                    return 0
                
        if len (self.recentPosts) > 15:
            self.recentPosts = self.recentPosts [1:]

        if not self.posts:
            self.posts = []
        self.posts.append (post)
        if self.listener:
            self.listener (post)


    def createPost (self, k, v):
        args = v.split ("\0")
        args.append (int (k))
        return Post (*args)

    def needsUpdate (self):
        """Returns true if the feed should be updated from its RSS source"""
        return self.lastchecked + self.interval < time.time ()
        
